#version 430

#define PI 3.1415926535897932384626433832795

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;


layout(std140, binding = 1) uniform projectionData
{
    mat4 viewProjection;    // size = 64B
    vec3 cameraPosition;    // size = 16B
};

uniform vec3 color;
uniform mat4 modelMatrix;

uniform float height;
uniform float wavelength;
uniform float waveAmplitude;
uniform float time;
uniform bool watermeshAnimation;


float generateOffset(float x, float z, float wavelength, float waveAmplitude, float time){
    float radiansX = (x/wavelength + time) * 2.0 * PI;
    float radiansZ = (z/wavelength + time) * 2.0 * PI;
    return waveAmplitude * 0.5 * (sin(radiansZ) + cos(radiansX));
}

vec3 applayDistortion(vec3 currentVertex, float wavelength, float waveAmplitude, float time){
    float distortion = generateOffset(currentVertex.x,currentVertex.z, wavelength, waveAmplitude, time);
    return currentVertex + vec3(distortion,distortion,distortion);
}

void main() {

    //float height = 0.6; //sin(position.x * 3)*2

    vec3 newPosition = position;
    newPosition = newPosition + height * normal;
    newPosition = applayDistortion(newPosition, wavelength, waveAmplitude, time);

    gl_Position = viewProjection * modelMatrix * vec4(newPosition, 1);
}

